<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.mergado.cz
 * @since      1.0.0
 *
 * @package    Mergado_Marketing_Pack
 * @subpackage Mergado_Marketing_Pack/admin
 */

use Mergado\Tools\ImportPricesClass;
use Mergado\Tools\RssClass;
use Mergado\Tools\Settings;
use Mergado\Tools\XMLCategoryFeed;
use Mergado\Tools\XMLClass;
use Mergado\Tools\XMLProductFeed;
use Mergado\Tools\XMLStockFeed;

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Mergado_Marketing_Pack
 * @subpackage Mergado_Marketing_Pack/admin
 * @author     Mergado technologies, s. r. o. <info@mergado.cz>
 */

require_once( ABSPATH . 'wp-admin/includes/plugin.php' ); // Imported because of is_plugin_active usage

if (!is_plugin_active( 'woocommerce/woocommerce.php')) {
    deactivate_plugins(plugin_basename( __MERGADO_BASE_FILE__ ));
    die(WOOCOMMERCE_DEPENCENCY_MESSAGE);
} else {
    if (!defined('WC_ABSPATH')) {
        define('WC_ABSPATH', dirname(__FILE__) . '/../../woocommerce/');
    }
    include_once( WC_ABSPATH . '/includes/export/class-wc-product-csv-exporter.php' );
    if (!class_exists('WC_CSV_Batch_Exporter', false)) {
        include_once( WC_ABSPATH . '/includes/export/abstract-wc-csv-batch-exporter.php' );
    }
    if (!class_exists('WC_CSV_Exporter', false)) {
        include_once( WC_ABSPATH . '/includes/export/abstract-wc-csv-exporter.php' );
    }
}

require_once __MERGADO_TOOLS_DIR__ . 'RssClass.php';
require_once __MERGADO_TOOLS_DIR__ . 'XMLClass.php';
require_once __MERGADO_TOOLS_XML_DIR__ . 'XMLStockFeed.php';
require_once __MERGADO_TOOLS_XML_DIR__ . 'XMLCategoryFeed.php';
require_once __MERGADO_TOOLS_XML_DIR__ . 'XMLProductFeed.php';


class Megrado_export extends WC_Product_CSV_Exporter {

    public function generate_data($start = 0, $limit = null) {

        if($limit === NULL) {
            $limit = 9999999;
        }

        $this->set_page($start);
        $this->set_limit($limit);
        $this->prepare_data_to_export();
        return $this->row_data;
    }

}

class Mergado_Marketing_Pack_Admin {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

        $cronRss = new RssClass();
        $cronRss->getFeed();

        self::init_hooks();
    }

    public static function init_hooks() {
        add_action('admin_menu', array('Mergado_Marketing_Pack_Admin', 'admin_menu'));
    }

    public static function admin_menu() {
        self::load_menu();
    }

    public static function load_menu() {
        // Add main menu pages
        add_menu_page(__('Mergado Pack', 'mergado-marketing-pack'), __('Mergado', 'mergado-marketing-pack'), 'manage_options', 'mergado-config', array('Mergado_Marketing_Pack_Admin', 'display'), 'dashicons-chart-line', 58);
        add_submenu_page('woocommerce', __('Mergado Pack', 'mergado-marketing-pack'), __('Mergado Pack', 'mergado-marketing-pack'), 'manage_options', 'mergado-config', array('Mergado_Marketing_Pack_Admin', 'display'));

        // Add submenu pages
        add_submenu_page('mergado-config', __('Settings', 'mergado-marketing-pack'), __('Settings', 'mergado-marketing-pack'), 'manage_options', 'mergado-settings', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', __('Cron tasks', 'mergado-marketing-pack'), __('Cron tasks', 'mergado-marketing-pack'), 'manage_options', 'mergado-cron', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', __('XML feeds', 'mergado-marketing-pack'), __('XML feeds', 'mergado-marketing-pack'), 'manage_options', 'mergado-feed', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', __('Ad Systems', 'mergado-marketing-pack'), __('Ad Systems', 'mergado-marketing-pack'), 'manage_options', 'mergado-adsys', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', __('Support', 'mergado-marketing-pack'), __('Support', 'mergado-marketing-pack'), 'manage_options', 'mergado-support', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', __('News', 'mergado-marketing-pack'), __('News', 'mergado-marketing-pack'), 'manage_options', 'mergado-news', array('Mergado_Marketing_Pack_Admin', 'display'));
        add_submenu_page('mergado-config', __('Licence', 'mergado-marketing-pack'), __('Licence', 'mergado-marketing-pack'), 'manage_options', 'mergado-licence', array('Mergado_Marketing_Pack_Admin', 'display'));
    }

    public static function display() {
        global $wp;
        $token = self::getToken();
        $currentBlogId = Settings::getCurrentBlogId();

        if (isset($_GET['page'])) {
            $page = $_GET['page'];
            if (isset($_GET['action'])) {
                if (isset($_GET['token']) && $_GET['token'] = $token && $token != '' && $token != NULL) {
                    $action = $_GET['action'];
                    switch ($action) {
                        case 'productDownload':
                            $token = self::getToken();
                            $file = __MERGADO_XML_DIR__ . $currentBlogId . '/' . 'products_' . $token . '.xml';
                            XMLClass::download($page, $file);
                            break;
                        case 'productDelete':
                            $token = self::getToken();
                            $file = __MERGADO_XML_DIR__ . $currentBlogId . '/' . 'products_' . $token . '.xml';
                            unlink($file);

                            $xmlProductFeed = new XMLProductFeed();
                            XMLClass::setFeedCount('mergado-feed-iterator', 0);
                            XMLClass::deleteTemporaryFiles($xmlProductFeed->productFeedTmpDir);

                            wp_redirect('admin.php?page=' . 'mergado-feed&flash=productDeleted');
                            break;
                        case 'productStartAgain':
                            $files = glob(__MERGADO_TMP_DIR__ . Settings::getCurrentBlogId() . '/productFeed/*');

                            foreach ($files as $file) {
                                if (is_file($file)) {
                                    unlink($file);
                                }
                            }

                            wp_redirect('admin.php?page=' . 'mergado-feed&flash=productStartedAgain');
                            break;

                        case 'categoryStartAgain':
                            $files = glob(__MERGADO_TMP_DIR__ . Settings::getCurrentBlogId() . '/categoryFeed/*');

                            foreach ($files as $file) {
                                if (is_file($file)) {
                                    unlink($file);
                                }
                            }

                            wp_redirect('admin.php?page=' . 'mergado-feed&flash=categoryStartedAgain');
                            break;

                        case 'stockStartAgain':
                            $files = glob(__MERGADO_TMP_DIR__ . Settings::getCurrentBlogId() . '/stockFeed/*');

                            foreach ($files as $file) {
                                if (is_file($file)) {
                                    unlink($file);
                                }
                            }

                            wp_redirect('admin.php?page=' . 'mergado-feed&flash=stockStartedAgain');
                            break;
                        case 'stockDownload':
                            $token = self::getToken();
                            $file = __MERGADO_XML_DIR__ . $currentBlogId . '/' . 'stock_' . $token . '.xml';
                            XMLClass::download($page, $file);
                            break;
                        case 'stockDelete':
                            $token = self::getToken();
                            $file = __MERGADO_XML_DIR__ . $currentBlogId . '/' . 'stock_' . $token . '.xml';
                            unlink($file);

                            $xmlStockFeed = new XMLStockFeed();
                            XMLClass::setFeedCount('mergado-product-stock-iterator', 0);
                            XMLClass::deleteTemporaryFiles($xmlStockFeed->stockFeedTmpDir);

                            wp_redirect('admin.php?page=' . 'mergado-feed&flash=stockDeleted');
                            break;
                        case 'categoryDownload':
                            $token = self::getToken();
                            $file = __MERGADO_XML_DIR__ . $currentBlogId . '/' . 'category_' . $token . '.xml';
                            XMLClass::download($page, $file);
                            break;
                        case 'categoryDelete':
                            $token = self::getToken();
                            $file = __MERGADO_XML_DIR__ . $currentBlogId . '/' . 'category_' . $token . '.xml';
                            unlink($file);

                            $xmlCategoryFeed = new XMLCategoryFeed();
                            XMLClass::setFeedCount('mergado-product-category-iterator', 0);
                            XMLClass::deleteTemporaryFiles($xmlCategoryFeed->categoryFeedTmpDir);

                            wp_redirect('admin.php?page=' . 'mergado-feed&flash=categoryDeleted');
                            break;
                        case 'productCron':
                            $xmlProductFeed = new XMLProductFeed();
                            $xmlProductFeed->cron($page, true);
                            break;
                        case 'stockCron':
                            $xmlStockFeed = new XMLStockFeed();
                            $xmlStockFeed->generateStockXML($page, true);
                            break;
                        case 'categoryCron':
                            $xmlCategoryFeed = new XMLCategoryFeed();
                            $xmlCategoryFeed->generateCategoryXML($page, true);
                            break;
                        case 'importPrices':
                            $importPrices = new ImportPricesClass();
                            $importPrices->importPrices($page, true);
                            break;
                        default:
                            exit;
                    }
                } else {
                    wp_redirect('admin.php?page=' . $page);
                }
            } else {
                switch ($page) {
                    case 'mergado-settings':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-settings.php');
                        break;
                    case 'mergado-config':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-info.php');
                        break;
                    case 'mergado-cron':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-cron.php');
                        break;
                    case 'mergado-feed':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-feed.php');
                        break;
                    case 'mergado-licence':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-licence.php');
                        break;
                    case 'mergado-support':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-support.php');
                        break;
                    case 'mergado-adsys':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-adsys.php');
                        break;
                    case 'mergado-news':
                        require_once (__DIR__ . '/partials/template-mergado-marketing-pack-display-news.php');
                        break;
                    default:
                        exit;
                        break;
                }
            }
        } else {
            die;
        }
    }

    /*******************************************************************************************************************
     * ENQUEUE
     *******************************************************************************************************************/

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Mergado_Marketing_Pack_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Mergado_Marketing_Pack_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/mergado-marketing-pack-admin.css', array(), $this->version, 'all');
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Mergado_Marketing_Pack_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Mergado_Marketing_Pack_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/mergado-marketing-pack-admin.js', array('jquery'), $this->version, false);
        wp_enqueue_script($this->plugin_name . 'iframe', plugin_dir_url(__FILE__) . 'vendors/iframe-resizer/js/iframeResizer.min.js', array('jquery'), $this->version, true);
        wp_enqueue_script($this->plugin_name . 'iframe-enabler', plugin_dir_url(__FILE__) . 'js/mergado-marketing-pack-admin-iframe.js', array('jquery'), $this->version, true);
    }

    /*******************************************************************************************************************
     * TOKEN
     *******************************************************************************************************************/

    public static function generateToken() {
        $token = sha1(get_site_url() . 'MMP2017WP' . uniqid());
        update_option('mmp_token', $token);
        return $token;
    }

    public static function getToken() {
        $token = get_option('mmp_token');

        if ($token === NULL || $token === '' || !$token) {
            $token = self::generateToken();
        }

        return $token;
    }

    public static function deleteToken() {
        delete_option('mmp_token');
    }

    /*******************************************************************************************************************
     * CREATE DIR
     *******************************************************************************************************************/

    public static function createDir($dir)
    {
        if (!is_dir($dir)) {
            mkdir($dir, 0777, true);
        }
    }

    public static function checkAndCreateTmpDataDir()
    {
        if(!is_dir(__MERGADO_TMP_DIR__)) {
            mkdir(__MERGADO_TMP_DIR__);
        }

        if(!is_dir(__MERGADO_XML_DIR__)) {
            mkdir(__MERGADO_XML_DIR__);
        }
    }
}
