<?php

/**
 * NOTICE OF LICENSE.
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    www.mergado.cz
 * @copyright 2016 Mergado technologies, s. r. o.
 * @license   LICENSE.txt
 */

namespace Mergado\Tools;

use DateTime;
use Mergado_Marketing_Pack_Admin;
use WP_Query;
use XMLWriter;

class XMLCategoryFeed
{
    protected $language;
    protected $currency;
    protected $logger;
    protected $token;
    protected $lock;

    protected $currentBlogId;

    // Dirs
    public $tmpBlogDir;
    public $categoryFeedTmpDir;
    public $outputBlogDir;
    public $categoryFeedOutputDir;

    /*******************************************************************************************************************
     * XML GENERATORS
    *******************************************************************************************************************/
    /*******************************************************************************************************************
     * CATEGORY FEED
     *******************************************************************************************************************/

    public function __construct()
    {
        $this->logger = wc_get_logger();
        $this->token = Mergado_Marketing_Pack_Admin::getToken();
        $this->currentBlogId = Settings::getCurrentBlogId();
        $this->lock = get_option('category_feed_' . $this->token);

        $this->tmpBlogDir = __MERGADO_TMP_DIR__ . $this->currentBlogId;
        $this->categoryFeedTmpDir = __MERGADO_TMP_DIR__ . $this->currentBlogId . '/categoryFeed/';
        $this->outputBlogDir =  __MERGADO_XML_DIR__ . $this->currentBlogId;
        $this->categoryFeedOutputDir =  __MERGADO_XML_DIR__ . $this->currentBlogId . '/';
    }

    /**
     * @param $page
     * @param bool $redirect
     * @return bool
     */
    public function generateCategoryXML($page, $redirect = false)
    {
        $now = new DateTime();
        $this->createNecessaryDirs();

        if($this->isFeedLocked($now)) {
            $this->logger->info('CATEGORY FEED LOCKED - generating process can\'t proceed');
            throw new CronRunningException();
        } else {
            $this->setFeedLocked($now);

            $categoriesPerStep = XMLClass::getProductsPerStep('feed-form-category');
            $currentFilesCount = XMLClass::getCurrentTempFilesCount($this->categoryFeedTmpDir);
            $start = XMLClass::getStart($currentFilesCount);

            // If no temporary files, reset generating
            if ($start === 1) {
                $this->resetFeedGenerating();
            }

            $categoryList = $this->getCategories($start, $categoriesPerStep);

            // Step generating
            if ($this->isPartial($categoriesPerStep, $categoryList)) {
                $file = $this->categoryFeedTmpDir . ($currentFilesCount) . '.xml';

                $this->logger->info('Mergado log: Category feed generator started - step ' . $currentFilesCount);
                $this->createXML($file, $start, $categoriesPerStep, $categoryList);
                $this->logger->info('Mergado log: Category feed generator ended - step ' . $currentFilesCount);
                $this->logger->info('Mergado log: Category feed generator saved XML file - step ' . $currentFilesCount);

                $this->increaseIterator();
                $this->unlockFeed();

                if ($redirect) {
                    wp_redirect('admin.php?page=' . $page . '&flash=stepGenerated');
                    exit;
                }
            // Normal generating
            } else if ($this->isNormal($categoriesPerStep, $categoryList)) {
                $file = $this->categoryFeedOutputDir . 'category_' . $this->token . '.xml';

                $this->logger->info('Mergado log: Category feed generator started');
                $this->createXML($file);
                $this->logger->info('Mergado log: Category feed generator ended');
                $this->logger->info('Mergado log: Category feed generator saved XML file');

                $this->unlockFeed();

                if ($redirect) {
                    wp_redirect('admin.php?page=' . $page . '&flash=categoryGenerated');
                    exit;
                }
            // Merge
            } else {
                $this->mergeTemporaryFiles();
                $this->unlockFeed();
                $this->updateFeedCount();

                if ($redirect) {
                    wp_redirect('admin.php?page=' . $page . '&flash=mergeGenerated');
                    exit;
                }
            }

            return true;
        }
    }


    /**
     * @param $file
     * @param null $start
     * @param null $limit
     * @param null $categories
     * @return bool
     */
    private function createXML($file, $start = null, $limit = null, $categories = null)
    {
        if ($categories === null) {
            $categories = $this->getCategories($start, $limit);
        }

        $xml_new = new XMLWriter();
        $xml_new->openURI($file);
        $xml_new->startDocument('1.0', 'UTF-8');
        $xml_new->startElement('CHANNEL');
        $xml_new->writeAttribute('xmlns', 'http://www.mergado.com/ns/category/1.7');

        $xml_new->startElement('LINK');
        $xml_new->text(get_home_url(get_current_blog_id()));
        $xml_new->endElement();

        $xml_new->startElement('GENERATOR');
        $xml_new->text('mergado.woocommerce.marketingpack.' . str_replace('.', '_', PLUGIN_VERSION));
        $xml_new->endElement();

        foreach ($categories as $cat) {
            $minPrice = self::getCategoryPrice($cat->slug, 'ASC');
            $maxPrice = self::getCategoryPrice($cat->slug, 'DESC');

            $taxRate = Settings::getTaxRatesForCountry(get_option(Settings::VAT), '');

            if(Settings::isTaxCalculated()) {
                if(Settings::isTaxIncluded()) {
                    $catMinPrice = $minPrice;
                    $catMaxPrice = $maxPrice;
                } else {
                    $catMinPrice = round($minPrice * (1 + ($taxRate / 100)), 2);
                    $catMaxPrice = round($maxPrice * (1 + ($taxRate / 100)), 2);
                }
            } else {
                $catMinPrice = $minPrice;
                $catMaxPrice = $maxPrice;
            }

            if($cat->parent !== 0) {
                $breadcrumbs = self::getBreadcrumbs($cat->parent, $cat->name);
            } else {
                $breadcrumbs = $cat->name;
            }

            // START ITEM
            $xml_new->startElement('ITEM');

            $xml_new->startElement('CATEGORY_NAME');
            $xml_new->text('<![CDATA[' . $cat->name . ']]>');
            $xml_new->endElement();

            $xml_new->startElement('CATEGORY');
            $xml_new->text('<![CDATA[' . $breadcrumbs . ']]>');
            $xml_new->endElement();

            $xml_new->startElement('CATEGORY_ID');
            $xml_new->text($cat->term_id);
            $xml_new->endElement();

            $xml_new->startElement('CATEGORY_URL');
            $xml_new->text(get_category_link($cat));
            $xml_new->endElement();

            $xml_new->startElement('CATEGORY_QUANTITY');
            $xml_new->text($cat->count);
            $xml_new->endElement();

            $xml_new->startElement('CATEGORY_DESCRIPTION');
            $xml_new->text('<![CDATA[' . $cat->description . ']]>');
            $xml_new->endElement();

            $xml_new->startElement('CATEGORY_MIN_PRICE_VAT');
            $xml_new->text($catMinPrice);
            $xml_new->endElement();

            $xml_new->startElement('CATEGORY_MAX_PRICE_VAT');
            $xml_new->text($catMaxPrice);
            $xml_new->endElement();

            // END ITEM
            $xml_new->endElement();
        }

        $xml_new->endElement();
        $xml_new->endDocument();
        $xml_new->flush();
        unset($xml_new);

        return true;
    }

    /**
     * Return max or min price in category
     *
     * SORT -> 'DESC' for MAX value
     * SORT -> 'ASC' for MIN value
     *
     * @param $slug
     * @param $sort
     * @return mixed
     */
    private static function getCategoryPrice($slug, $sort)
    {
        $args = array(
            'posts_per_page' => 1,
            'post_type' => 'product',
            'orderby' => 'meta_value_num',
            'order' => $sort,
            'tax_query' => array(
                array(
                    'taxonomy' => 'product_cat',
                    'field' => 'slug',
                    'terms' => $slug,
                    'operator' => 'IN'
                )
            ),
            'meta_query' => array(
                array(
                    'key' => '_price',
                )
            )
        );

        $loop = new WP_Query($args);

        return get_post_meta($loop->posts[0]->ID, '_price', true);
    }

    /**
     * Return breadcrumbs for category feed
     *
     * @param $id
     * @param $name
     * @return string
     */
    private static function getBreadcrumbs($id, $name)
    {
        $term = get_term_by( 'id', $id, 'product_cat' );

        if($term->parent != 0) {
            $newName = $term->name . ' | ' . $name;
            return self::getBreadcrumbs($term->parent, $newName);
        }

        return $term->name . ' | ' . $name;
    }

    /*******************************************************************************************************************
     * FEED OPTIONS
     *******************************************************************************************************************/

    /**
     * Merge files, create XML and delete temporary files
     */
    private function mergeTemporaryFiles()
    {
        $storage = $this->categoryFeedOutputDir . 'category_' . $this->token . '.xml';
        $tmpShopDir = $this->categoryFeedTmpDir;

        $logger = wc_get_logger();
        $logger->info('Merging Xml files of category feed.');
        $loop = 0;

        $xmlstr = '<CHANNEL xmlns="http://www.mergado.com/ns/category/1.7">';

        foreach (glob($tmpShopDir . '*.xml') as $file) {
            $xml = simplexml_load_file($file);
            $innerLoop = 0;
            foreach ($xml as $item) {
                if ($loop != 0 && (preg_match('/^mergado.woocommerce/', $item[0]) || ($innerLoop == 0 || $innerLoop == 1))) {
                    $innerLoop++;
                    continue;
                } else {
                    $innerLoop++;
                    $xmlstr .= $item->asXml();
                }
            }

            $loop++;
        }

        $xmlstr .= '</CHANNEL>';

        $xml_new = new XMLWriter();

        $xml_new->openURI($storage);
        $xml_new->startDocument('1.0', 'UTF-8');
        $xml_new->writeRaw($xmlstr);
        $xml_new->endDocument();

        $this->logger->info('Category feed merged. XML created.');


        XMLClass::deleteTemporaryFiles($this->categoryFeedTmpDir);

        return true;
    }

    private function isPartial($stepProducts, $productsList)
    {
        return XMLClass::isPartial($stepProducts, $productsList);
    }

    private function isNormal($stepProducts, $productsList)
    {
        return XMLClass::isNormal($stepProducts, $productsList);
    }

    /**
     * Reset feed and delete all TMP files
     */
    private function resetFeedGenerating()
    {
        XMLClass::resetFeedGenerating('mergado-category-feed-iterator', $this->categoryFeedTmpDir);
    }

    /*******************************************************************************************************************
     * FEED LOCKING
     *******************************************************************************************************************/

    /**
     * Return if feed is currently locked
     *
     * @param $now
     * @return bool
     */
    private function isFeedLocked($now)
    {
        return XMLClass::isFeedLocked($this->lock, $now);
    }

    /**
     * Lock feed
     * @param $now
     */
    private function setFeedLocked( $now)
    {
        XMLClass::setFeedLocked('category_feed_' . $this->token, $now);
        $this->logger->info('CATEGORY FEED - locking');
    }

    /**
     * Unlock feed
     */
    private function unlockFeed() {
        XMLClass::unlockFeed('category_feed_' . $this->token);
        $this->logger->info('CATEGORY FEED - unlocking');
    }

    /*******************************************************************************************************************
     * FEED PRODUCT COUNT
     *******************************************************************************************************************/

    /**
     * Save the feed count to database for next runs
     */
    private function updateFeedCount()
    {
        update_option('mergado-category-feed-count', (int)get_option('mergado-category-feed-iterator'));
    }

    private function increaseIterator()
    {
        update_option('mergado-category-feed-iterator', (int)get_option('mergado-category-feed-iterator') + 1);
    }

    /*******************************************************************************************************************
     * FEED FOLDER MANIPULATION
     *******************************************************************************************************************/

    /**
     * Check and create necessary directories for this cron
     */
    private function createNecessaryDirs()
    {
        Mergado_Marketing_Pack_Admin::checkAndCreateTmpDataDir();
        Mergado_Marketing_Pack_Admin::createDir($this->tmpBlogDir);
        Mergado_Marketing_Pack_Admin::createDir($this->categoryFeedTmpDir);
        Mergado_Marketing_Pack_Admin::createDir($this->outputBlogDir);
        Mergado_Marketing_Pack_Admin::createDir($this->categoryFeedOutputDir);
    }

    /*******************************************************************************************************************
     * GET PRODUCTS
     *******************************************************************************************************************/

    /**
     * @param $start
     * @param $stepProducts
     */
    public function getCategories($start, $stepProducts)
    {
        return get_terms(
            [
                'taxonomy' => 'product_cat',
                'offset' => ($start - 1) * $stepProducts,
                'number' => $stepProducts,
                'hide_empty' => true
            ]
        );
    }

    /*******************************************************************************************************************
     * DATA FOR TEMPLATES
     *******************************************************************************************************************/

    public static function getDataForTemplates()
    {
        if((int)get_option('feed-form-category', 0) === 0) {
            $totalFiles = 0;
        } else {
            $productsCount = (int)get_option('mergado-category-feed-count');
            $productsCount = $productsCount === 0 ? wp_count_posts('product')->publish : $productsCount;

            $totalFiles = ceil($productsCount / (int)get_option('feed-form-category', 0));
        }

        $currentStep = XMLClass::getCurrentTempFilesCount(__MERGADO_TMP_DIR__ . Settings::getCurrentBlogId() . '/categoryFeed/');
        $stepProducts = (int) get_option('feed-form-category', 0);

        if ($totalFiles == 0) {
            $percentageStep = 0;
        } else {
            $percentageStep = round(($currentStep / ($totalFiles)) * 100, 2);
        }

        $isPartiallyGenerated =  (isset($totalFiles) && $totalFiles != 0 && $totalFiles != '');

        return [
            'totalFiles' => $totalFiles,
            'currentStep' => $currentStep,
            'stepProducts' => $stepProducts,
            'percentageStep' => $percentageStep,
            'isPartiallyGenerated' => $isPartiallyGenerated
        ];
    }
}
