<?php

/**
 * NOTICE OF LICENSE.
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    www.mergado.cz
 * @copyright 2016 Mergado technologies, s. r. o.
 * @license   LICENSE.txt
 */

namespace Mergado\Tools;

use Exception;
use Megrado_export;
use XMLWriter;

class XMLClass
{
    protected $language;
    protected $currency;

    /*******************************************************************************************************************
     * XML GENERATORS
    *******************************************************************************************************************/
    /*******************************************************************************************************************
     * CREATE XML
     *******************************************************************************************************************/

    public static function getProducts($start = 0, $limit = 999999999)
    {
        $exporter = new Megrado_export();

        $products = [];

        foreach ($exporter->generate_data($start, $limit) as $k => $v) {
            $parentId = wc_get_product( $v['id'] )->get_parent_id();
            $product = wc_get_product( $v['id'] );
            $productPublished = $product->get_status() === 'publish';

            if($parentId != 0) {
                $parentProduct = wc_get_product( $parentId );
                $parentPublished = $parentProduct->get_status() === 'publish';
            } else {
                $parentProduct = false;
                $parentPublished = true;
            }

            // Check if parent product exist (woocommerce made error - deleted main product but not variations)
            // Variation is for VARIATION, VARIABLE is for main VARIATION product
            if ($product->is_type('variation') && $parentProduct == false) {
                $parentExists = false;
            } else {
                $parentExists = true;
            }

            // Check if not password protected and if parent no password protected (for variants)
            if(!post_password_required($v['id']) && !post_password_required($parentId) && $parentPublished && $productPublished && $parentExists) {
                if ($v['type'] != 'grouped' && $v['published'] === 1) { // preskoceni slozenych produktu
                    $products[$v['id']] = $v;
                }
            }
        }


        return $products;
    }


    /*******************************************************************************************************************
     * DOWNLOAD
     *******************************************************************************************************************/

    public static function download($page, $file) {
        $logger = wc_get_logger();
        if (file_exists($file)) {
            $logger->info('Mergado log: XML product file was downloaded');
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename="' . basename($file) . '"');
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($file));
            ob_clean();
            flush();
            echo readfile($file);
            exit;
        } else {
            wp_redirect('admin.php?page=' . $page);
        }
    }


    /*******************************************************************************************************************
     * FIND
     *******************************************************************************************************************/

    protected static function findParams($product, $values, $parent, $parentValues) {
        $usedParams = []; // For checking if already added
        $params = [];

        //Default product attributes
        foreach ($values as $attrName => $attrValue) {
            $item = [];

            if($attrValue instanceof \WC_Product_Attribute) {
//                $item['name'] = wc_attribute_label($attrValue->get_name()); //Case sensitive
                $item['name'] = wc_attribute_taxonomy_slug($attrName); //Lowercase // cant get simple products

                if(substr( $attrValue->get_name(), 0, 3 ) === "pa_") {
                    $item['value'] = $product->get_attribute( $attrValue->get_name() );
                } else {
                    $item['value'] = implode(', ', $attrValue->get_options());
                }
            } else {
                $item['name'] = wc_attribute_taxonomy_slug($attrName);
                $item['value'] = $attrValue;
            }

            if($item['value'] !== '') {
                $params[] = $item;
                $usedParams[] = $item['name'];
            }
        }

        //Other parameters/attributes not used in variation
        if($parentValues) {
            foreach ($parentValues as $attrName => $attrValue) {
                $item = [];

                if ($attrValue instanceof \WC_Product_Attribute) {
//                    $item['name'] = wc_attribute_label($attrValue->get_name()); //Case //sensitive
                    $item['name'] = wc_attribute_taxonomy_slug($attrName); //Lowercase // cant get simple products

                    if(substr( $attrValue->get_name(), 0, 3 ) === "pa_") {
                        $item['value'] = $parent->get_attribute( $attrValue->get_name() );
                    } else {
                        $item['value'] = implode(', ', $attrValue->get_options());
                    }

                } else {
                    $item['name'] = wc_attribute_taxonomy_slug($attrName);
                    $item['value'] = $attrValue;
                }


                if (!in_array($item['name'], $usedParams)) {
                    $params[] = $item;
                }
            }
        }

        return $params;
    }

    /*******************************************************************************************************************
     * GET FILES COUNT
     *******************************************************************************************************************/

    /**
     * @param $dir
     * @return int
     */
    public static function getCurrentTempFilesCount($dir)
    {
        if (glob($dir . '*.xml') != false) {
            return count(glob($dir . '*.xml'));
        } else {
            return 0;
        }
    }

    /*******************************************************************************************************************
     * PRODUCT COUNT
     *******************************************************************************************************************/

    /**
     * @param $feedName
     * @return int
     */
    public static function getProductsPerStep($feedName)
    {
        return (int)get_option($feedName, 0);
    }

    /**
     * @param $currentFilesCount
     * @return int
     */
    public static function getStart($currentFilesCount)
    {
        return $currentFilesCount === 0 ? 1 : $currentFilesCount + 1;
    }

    public static function isPartial($stepProducts, $productsList)
    {
        return $stepProducts !== 0 && $productsList !== [];
    }

    public static function isNormal($stepProducts, $productsList)
    {
        return $stepProducts === 0 || $stepProducts === false || ($productsList && ($stepProducts >= count($productsList)));
    }

    /**
     * Reset feed and delete all TMP files
     * @param $iteratorName
     * @param $tmpDir
     */
    public static function resetFeedGenerating($iteratorName, $tmpDir)
    {
        self::resetFeedCount($iteratorName);
        self::deleteTemporaryFiles($tmpDir);
    }

    /**
     * Delete all files from TMP folder
     * @param $tmpDir
     */
    public static function deleteTemporaryFiles($tmpDir)
    {
        $files = glob($tmpDir . '*');

        foreach ($files as $file) {
            if (is_file($file)) {
                unlink($file);
            }
        }
    }

    /**
     * Return if feed is currently locked
     *
     * @param $lock
     * @param $now
     * @return bool
     */
    public static function isFeedLocked($lock, $now)
    {
        if($lock && $lock !== 0 && $lock >= $now) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Lock feed
     * @param $feedName
     * @param $now
     */
    public static function setFeedLocked($feedName, $now)
    {
        update_option($feedName, $now->modify( "+1 minute" ));
    }

    /**
     * Unlock feed
     * @param $feedName
     */
    public static function unlockFeed($feedName) {
        update_option($feedName, 0);
    }

    public static function setFeedCount($feedIteratorName, $value) {
        update_option($feedIteratorName, $value);
    }

    /**
     * Reset feed count
     * @param $feedIteratorName
     */
    public static function resetFeedCount($feedIteratorName)
    {
        update_option($feedIteratorName, 1);
    }
}

/**
 * Thrown when an service returns an exception
 */
class CronRunningException extends Exception
{

};
